<?php

namespace NoccyLabs\React\Http\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use React\Http\Message\Response;
use NoccyLabs\React\Http\Security\Exception\SecurityException;
use NoccyLabs\React\Http\Security\UserManagerInterface;
use React\Promise\PromiseInterface;

class SecurityMiddleware
{
    public function __construct(
        private UserManagerInterface $userManager,
    )
    {
        
    }

    public function __invoke(ServerRequestInterface $request, ?callable $next = null): ResponseInterface|PromiseInterface
    {

        $authorization = $request->getHeaderLine("Authorization");
        if ($authorization && str_contains($authorization, " ")) {
            [$type,$args] = explode(" ", $authorization, 2);
            switch (strtolower($type)) {
                case "basic":
                    $userpass = base64_decode($args);
                    if (str_contains($userpass,":")) {
                        [$username,$password] = explode(":",$userpass,2);
                        try {
                            $user = $this->userManager->getUserByUsername($username);
                        } catch (\Exception $e) {
                            return Response::json("Invalid user")->withStatus(Response::STATUS_FORBIDDEN);
                        }
                        $hash = $user->getPasswordHash();
                        $tokens = $user->getTokens();
                        if (!$hash || !$this->checkAuth($hash, $tokens, $password)) {
                            return Response::json("Bad request")->withStatus(Response::STATUS_FORBIDDEN);
                        }
                        $userId = $user->getId();
                    } else {
                        return Response::json("Bad request")->withStatus(Response::STATUS_BAD_REQUEST);
                    }
                    break;
                default:
                    return Response::json("Bad request")->withStatus(Response::STATUS_BAD_REQUEST);
            }
        }

        $token = $request->getHeaderLine('x-token');
        if ($token) {
            try {
                $user = $this->userManager->getUserByToken($token);
            }
            catch (SecurityException $e)
            {
                return Response::json("Access denied")->withStatus(Response::STATUS_UNAUTHORIZED);
            }
            $userId = $user->getId();
        }

        $session = $request->getAttribute("session");

        if ($session) {
            if (!isset($user) && isset($session['_uid'])) {
                $userId = $session['_uid'];
                try {
                    $user = $this->userManager->getUser($userId);
                }
                catch (SecurityException $e)
                {
                    return Response::json("Access denied")->withStatus(Response::STATUS_UNAUTHORIZED);
                }
            } elseif (isset($user) && !isset($session['_uid'])) {
                $session['_uid'] = $user->getId();
            }
        }

        $response = $next(
            $request
                ->withAttribute("authenticated", isset($user))
                ->withAttribute("user", $user??null)
                ->withAttribute("user_id", $userId??null)
        );

        if ($response instanceof ResponseInterface) {
            if ($session)
                return $response
                    ->withAddedHeader("set-Cookie", "sid={$session->getId()}; Path=/");
        }

        return $response;
    }

    public function checkAuth(string $hash, array $tokens, string $password): bool
    {
        if (str_starts_with($hash, '$2y$')) {
            if (true == password_verify($password, $hash)) return true;
        }
        foreach ($tokens as $token) {
            if ($token && $token == $password) {
                return true;
            }
        }
        return false;
    }
}
