<?php

namespace NoccyLabs\React\Http;

use NoccyLabs\React\Http\Middleware\SecurityMiddleware;
use NoccyLabs\React\Http\Security\StaticUser;
use NoccyLabs\React\Http\Security\StaticUserManager;
use NoccyLabs\React\Http\Security\UserManagerInterface;
use PHPUnit\Framework\Attributes\CoversClass;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use React\Http\Message\Response;
use React\Http\Message\ServerRequest;

#[CoversClass(SecurityMiddleware::class)]
class SecurityMiddlewareTest extends \PHPUnit\Framework\TestCase
{
    private function createUserManager(): UserManagerInterface
    {
        $userManager = new StaticUserManager();

        $userManager->createUser(
            new StaticUser(
                id: 1,
                username: 'user',
                passwordHash: password_hash('password', PASSWORD_BCRYPT),
                tokens: [ 'validtoken' ],
                roles: [ 'admin', 'user' ]
            )
        );

        return $userManager;
    }

    public function testLoggingInWithValidPassword()
    {
        $userManager =  $this->createUserManager();
        $middleware = new SecurityMiddleware($userManager);

        $request = new ServerRequest("GET", "/", [
            "Authorization" => "basic ".base64_encode("user:password")
        ]);
        $response = Response::plaintext("");
        
        $next = function (ServerRequestInterface $request) {
            $this->assertEquals(1, $request->getAttribute("user_id"));
            return Response::plaintext("");
        };

        $response = $middleware($request, $next);

        $this->assertInstanceOf(ResponseInterface::class, $response);
        $this->assertEquals(200, $response->getStatusCode());
    }

    public function testLoggingInWithInvalidPassword()
    {
        $userManager =  $this->createUserManager();
        $middleware = new SecurityMiddleware($userManager);

        $request = new ServerRequest("GET", "/", [
            "Authorization" => "basic ".base64_encode("user:wrongpassword")
        ]);
        $response = Response::plaintext("");
        
        $next = function (ServerRequestInterface $request) {
            $this->assertNull($request->getAttribute("user_id"));
            return Response::plaintext("");
        };

        $response = $middleware($request, $next);

        $this->assertInstanceOf(ResponseInterface::class, $response);
        $this->assertEquals(403, $response->getStatusCode());
    }

    public function testLoggingInWithTokenInHeader()
    {
        $userManager =  $this->createUserManager();
        $middleware = new SecurityMiddleware($userManager);

        $request = new ServerRequest("GET", "/", [
            "X-Token" => "validtoken"
        ]);
        $response = Response::plaintext("");
        
        $next = function (ServerRequestInterface $request) {
            $this->assertEquals(1, $request->getAttribute("user_id"));
            return Response::plaintext("");
        };

        $response = $middleware($request, $next);

        $this->assertInstanceOf(ResponseInterface::class, $response);
        $this->assertEquals(200, $response->getStatusCode());
    }

    public function testLoggingInWithTokenAsPassword()
    {
        $userManager =  $this->createUserManager();
        $middleware = new SecurityMiddleware($userManager);

        $request = new ServerRequest("GET", "/", [
            "Authorization" => "basic ".base64_encode("user:validtoken")
        ]);
        $response = Response::plaintext("");
        
        $next = function (ServerRequestInterface $request) {
            $this->assertEquals(1, $request->getAttribute("user_id"));
            return Response::plaintext("");
        };

        $response = $middleware($request, $next);

        $this->assertInstanceOf(ResponseInterface::class, $response);
        $this->assertEquals(200, $response->getStatusCode());
    }

}